import pytest

from windows_fonts import FontCollection


@pytest.fixture(scope="module")
def collection():
    return FontCollection()


def test_len(collection: FontCollection):
    assert len(collection) > 0


def test_get_index_key_same(collection: FontCollection):
    by_idx = collection[0]
    assert by_idx is not None

    by_key = collection[by_idx.name]
    assert by_key is not None

    assert by_idx == by_key


def test_no_such_font(collection: FontCollection):
    with pytest.raises(KeyError, match=r"unknown font family 'foobarbaznotfound'"):
        collection["foobarbaznotfound"]



from __future__ import annotations
import enum

class FontCollection:
    def __len__(self) -> int: ...
    def __getitem__(self, key: int | str) -> FontFamily: ...

class FontFamily:
    name: str
    def __len__(self) -> int: ...
    def __getitem__(self, idx: int) -> FontVariant: ...
    def get_matching_variants(
        self,
        *,
        weight: float | Weight | None = None,
        style: Style | None = None,
        width: float | None = None,
        slant: float | None = None,
        optical_size: float | None = None,
        italic: bool | None = None,
    ) -> list[FontVariant]: ...
    def get_best_variant(
        self,
        *,
        weight: float | Weight | None = None,
        style: Style | None = None,
        width: float | None = None,
        slant: float | None = None,
        optical_size: float | None = None,
        italic: bool | None = None,
    ) -> FontVariant: ...

class FontVariant:
    style: Style
    weight: Weight
    filename: str
    def files(self) -> list[str]: ...

class Style(enum.Enum):
    NORMAL = ...
    ITALIIC = ...
    OBLIQUE = ...

class Weight(enum.Enum):
    ULTRA_LIGHT = ...
    LIGHT = ...
    SEMI_LIGHT = ...
    REGULAR = ...
    MEDIUM = ...
    SEMI_BOLD = ...
    BOLD = ...
    EXTRA_BOLD = ...
    BLACK = ...
    EXTRA_BLACK = ...

def get_matching_variants(
    win32_family_names: str | None = None,
    typographic_family_names: str | None = None,
    full_name: str | None = None,
    postscript_name: str | None = None,
    weight_stretch_style_family_name: str | None = None,
    design_script_language_tag: str | None = None,
    supported_script_language_tag: str | None = None,
    preferred_family_names: str | None = None,
    preferred_subfamily_names: str | None = None,
) -> list[FontVariant]: ...



import collections.abc

import pytest

from windows_fonts import FontCollection, FontVariant, Style, Weight, get_matching_variants


@pytest.fixture(scope="module")
def collection():
    return FontCollection()


@pytest.fixture
def family(collection):
    return collection['Arial']


@pytest.fixture
def variant(family):
    return family[0]


def test_get_matching_variants(collection: FontCollection):
    vars = get_matching_variants(full_name="Arial Bold Italic")

    assert len(vars) == 1

    var = vars[0]
    assert var.style == Style.ITALIIC
    assert var.weight == Weight.BOLD
    assert var.name == "Bold Italic"
    assert var.family == collection["Arial"]


@pytest.mark.parametrize(
    ["kwargs", "match"],
    [
        [{"not_a_prop": "string"}, r"isn't a known font property name"],
        [{"copyright": "string"}, r"doesn't have a mapping to font property id"],
        [{"win32_family_names": 1.0}, r"'float' object cannot be converted to"],
        [{}, r"no filter conditions passed"],
    ],
)
def test_get_matching_variants_errors(kwargs, match: str, collection: FontCollection):
    with pytest.raises(TypeError, match=match):
        get_matching_variants(**kwargs)


def test_name(variant):
    assert variant.name == "Regular"


def test_repr(variant):
    rep = repr(variant)
    assert rep.startswith('<FontVariant name=Regular, family=<FontFamily name="Arial">,')


def test_filename(variant):
    variant.filename.lower().endswith("ARIAL.TTF")


def test_style(variant):
    assert isinstance(variant.style, Style)


def test_weight(variant):
    assert isinstance(variant.weight, Weight)


def test_information(variant: FontVariant):
    info = variant.information

    assert "copyright" in info
    assert 1 in info  # The constant for copyright, as defined by the DirectWrite enums
    assert 1.0 not in info

    assert "copyright" in info.keys()
    assert info["copyright"] in info.values()
    assert ("copyright", info["copyright"] in info.items())
    assert iter(info)
    assert len(info) > 1

    # Test the iterator works
    assert "copyright" in [*iter(info)]

    with pytest.raises(KeyError):
        info[0]

    with pytest.raises(KeyError):
        info['madeup']



import os

import pytest

from windows_fonts import FontCollection, FontFamily, Style, Weight


@pytest.fixture(scope="module")
def collection():
    return FontCollection()


@pytest.fixture
def family(collection: FontCollection):
    return collection['Arial']


@pytest.mark.parametrize(
    ["weight", "style", "expected_props"],
    [
        pytest.param(None, None, {"weight": Weight.REGULAR, "style": Style.NORMAL}, id="None,None"),
        pytest.param(700, None, {"weight": Weight.BOLD, "style": Style.NORMAL}, id="700,None"),
        pytest.param(Weight.BOLD, None, {"weight": Weight.BOLD, "style": Style.NORMAL}, id="BOLD,None"),
        pytest.param(Weight.BOLD, Style.ITALIIC, {"weight": Weight.BOLD, "style": Style.ITALIIC}, id="BOLD,True"),
    ],
)
def test_get_best_match(weight, style, expected_props, family: FontFamily):
    var = family.get_best_variant(weight=weight, style=style)

    for (name, val) in expected_props.items():
        assert getattr(var, name) == val


@pytest.mark.parametrize(
    ["width", "weight", "italic", "expected_props"],
    [
        pytest.param(
            100,
            None,
            None,
            {"weight": Weight.REGULAR, "style": Style.NORMAL, "file_stem": "ARIAL.TTF"},
            id="100,None,None",
        ),
        pytest.param(
            100,
            None,
            True,
            {"weight": Weight.REGULAR, "style": Style.ITALIIC, "file_stem": "ARIALI.TTF"},
            id="100,None,False",
        ),
        pytest.param(
            100,
            700,
            True,
            {"weight": Weight.BOLD, "style": Style.ITALIIC, "file_stem": "ARIALBI.TTF"},
            id="100,700,True",
        ),
        pytest.param(
            100,
            Weight.BOLD,
            True,
            {"weight": Weight.BOLD, "style": Style.ITALIIC, "file_stem": "ARIALBI.TTF"},
            id="100,BOLD,True",
        ),
    ],
)
def test_get_best_match_dwrite3(width, weight, italic, expected_props, family: FontFamily):
    """Test the _get_dwrite3_matching_variants code path"""
    var = family.get_best_variant(weight=weight, width=width, italic=italic)

    for (name, expected) in expected_props.items():
        if name == "file_stem":
            val = os.path.basename(var.filename).upper()
        else:
            val = getattr(var, name)
        assert val == expected


def test_get_matching_variants(family: FontFamily):
    variants = family.get_matching_variants()
    assert isinstance(variants, list)
    assert len(variants) > 2


def test_repr(family: FontFamily):
    assert repr(family) == '<FontFamily name="Arial">'


def test_len(family: FontFamily):
    # Lets just check it's an in int in a plausible range
    assert 2 < len(family) < 25
